#include <stdio.h>
#include <GL/gl.h>			
#include <GL/glu.h>
#include <math.h>

#define DIFF3(_a,_b,_c) { \
    (_c)[0] = (_a)[0] - (_b)[0]; \
    (_c)[1] = (_a)[1] - (_b)[1]; \
    (_c)[2] = (_a)[2] - (_b)[2]; \
}

void drawBox(GLfloat size, GLenum type)
{
   static GLfloat n[6][3] = {
      {-1.0, 0.0, 0.0},
      {0.0, 1.0, 0.0},
      {1.0, 0.0, 0.0},
      {0.0, -1.0, 0.0},
      {0.0, 0.0, 1.0},
      {0.0, 0.0, -1.0}
   };
   static GLint faces[6][4] = {
      {0, 1, 2, 3},
      {3, 2, 6, 7},
      {7, 6, 5, 4},
      {4, 5, 1, 0},
      {5, 6, 2, 1},
      {7, 4, 0, 3}
   };
   GLfloat v[8][3];
   GLint i;

   v[0][0] = v[1][0] = v[2][0] = v[3][0] = -size / 2;
   v[4][0] = v[5][0] = v[6][0] = v[7][0] = size / 2;
   v[0][1] = v[1][1] = v[4][1] = v[5][1] = -size / 2;
   v[2][1] = v[3][1] = v[6][1] = v[7][1] = size / 2;
   v[0][2] = v[3][2] = v[4][2] = v[7][2] = -size / 2;
   v[1][2] = v[2][2] = v[5][2] = v[6][2] = size / 2;

   for (i = 5; i >= 0; i--) {
      glBegin(type);
      glNormal3fv(&n[i][0]);
      glVertex3fv(&v[faces[i][0]][0]);
      glVertex3fv(&v[faces[i][1]][0]);
      glVertex3fv(&v[faces[i][2]][0]);
      glVertex3fv(&v[faces[i][3]][0]);
      glEnd();
   }
}

void glutSolidCube(GLdouble size)
{
   drawBox(size, GL_QUADS);
}

static GLUquadricObj *quadObj;

#define QUAD_OBJ_INIT() { if(!quadObj) initQuadObj(); }

static void initQuadObj(void)
{
   quadObj = gluNewQuadric();
   if (!quadObj)
     printf("gluNewQuadric: out of memory\n");
}

void glutSolidCone(GLdouble base, GLdouble height, GLint slices, GLint stacks)
{
   QUAD_OBJ_INIT();
   gluQuadricDrawStyle(quadObj, GLU_FILL);
   gluQuadricNormals(quadObj, GLU_SMOOTH);
   /* If we ever changed/used the texture or orientation state
      of quadObj, we'd need to change it to the defaults here
      with gluQuadricTexture and/or gluQuadricOrientation. */
   gluCylinder(quadObj, base, 0.0, height, slices, stacks);
}

void glutSolidTorus(GLfloat r, GLfloat R, GLint nsides, GLint rings) //Stolen right of the glut source code: (C) Mark J. Kilgard
{
   int i, j;
   GLfloat theta, phi, theta1;
   GLfloat cosTheta, sinTheta;
   GLfloat cosTheta1, sinTheta1;
   GLfloat ringDelta, sideDelta;

   ringDelta = 2.0 * M_PI / rings;
   sideDelta = 2.0 * M_PI / nsides;

   theta = 0.0;
   cosTheta = 1.0;
   sinTheta = 0.0;
   for (i = rings - 1; i >= 0; i--) {
      theta1 = theta + ringDelta;
      cosTheta1 = cos(theta1);
      sinTheta1 = sin(theta1);
      glBegin(GL_QUAD_STRIP);
      phi = 0.0;
      for (j = nsides; j >= 0; j--) {
	 GLfloat cosPhi, sinPhi, dist;

	 phi += sideDelta;
	 cosPhi = cos(phi);
	 sinPhi = sin(phi);
	 dist = R + r * cosPhi;

	 glNormal3f(cosTheta1 * cosPhi, -sinTheta1 * cosPhi, sinPhi);
	 glVertex3f(cosTheta1 * dist, -sinTheta1 * dist, r * sinPhi);
	 glNormal3f(cosTheta * cosPhi, -sinTheta * cosPhi, sinPhi);
	 glVertex3f(cosTheta * dist, -sinTheta * dist, r * sinPhi);
      }
      glEnd();
      theta = theta1;
      cosTheta = cosTheta1;
      sinTheta = sinTheta1;
   }
}

void glutSolidSphere(GLdouble radius, GLint slices, GLint stacks)
{
   QUAD_OBJ_INIT();
   gluQuadricDrawStyle(quadObj, GLU_FILL);
   gluQuadricNormals(quadObj, GLU_SMOOTH);
   /* If we ever changed/used the texture or orientation state
      of quadObj, we'd need to change it to the defaults here
      with gluQuadricTexture and/or gluQuadricOrientation. */
   gluSphere(quadObj, radius, slices, stacks);
}

static GLfloat dodec[20][3];

static void crossprod(GLfloat v1[3], GLfloat v2[3], GLfloat prod[3])
{
   GLfloat p[3];		/* in case prod == v1 or v2 */

   p[0] = v1[1] * v2[2] - v2[1] * v1[2];
   p[1] = v1[2] * v2[0] - v2[2] * v1[0];
   p[2] = v1[0] * v2[1] - v2[0] * v1[1];
   prod[0] = p[0];
   prod[1] = p[1];
   prod[2] = p[2];
}

static void normalize(GLfloat v[3])
{
   GLfloat d;

   d = sqrt(v[0] * v[0] + v[1] * v[1] + v[2] * v[2]);
   if (d == 0.0) {
      printf("normalize: zero length vector");
      v[0] = d = 1.0;
   }
   d = 1 / d;
   v[0] *= d;
   v[1] *= d;
   v[2] *= d;
}

static void pentagon(int a, int b, int c, int d, int e, GLenum shadeType)
{
   GLfloat n0[3], d1[3], d2[3];

   DIFF3(dodec[a], dodec[b], d1);
   DIFF3(dodec[b], dodec[c], d2);
   crossprod(d1, d2, n0);
   normalize(n0);

   glBegin(shadeType);
   glNormal3fv(n0);
   glVertex3fv(&dodec[a][0]);
   glVertex3fv(&dodec[b][0]);
   glVertex3fv(&dodec[c][0]);
   glVertex3fv(&dodec[d][0]);
   glVertex3fv(&dodec[e][0]);
   glEnd();
}


static void initDodecahedron(void)
{
   GLfloat alpha, beta;

   alpha = sqrt(2.0 / (3.0 + sqrt(5.0)));
   beta = 1.0 + sqrt(6.0 / (3.0 + sqrt(5.0)) -
		     2.0 + 2.0 * sqrt(2.0 / (3.0 + sqrt(5.0))));
    /* *INDENT-OFF* */
    dodec[0][0] = -alpha; dodec[0][1] = 0; dodec[0][2] = beta;
    dodec[1][0] = alpha; dodec[1][1] = 0; dodec[1][2] = beta;
    dodec[2][0] = -1; dodec[2][1] = -1; dodec[2][2] = -1;
    dodec[3][0] = -1; dodec[3][1] = -1; dodec[3][2] = 1;
    dodec[4][0] = -1; dodec[4][1] = 1; dodec[4][2] = -1;
    dodec[5][0] = -1; dodec[5][1] = 1; dodec[5][2] = 1;
    dodec[6][0] = 1; dodec[6][1] = -1; dodec[6][2] = -1;
    dodec[7][0] = 1; dodec[7][1] = -1; dodec[7][2] = 1;
    dodec[8][0] = 1; dodec[8][1] = 1; dodec[8][2] = -1;
    dodec[9][0] = 1; dodec[9][1] = 1; dodec[9][2] = 1;
    dodec[10][0] = beta; dodec[10][1] = alpha; dodec[10][2] = 0;
    dodec[11][0] = beta; dodec[11][1] = -alpha; dodec[11][2] = 0;
    dodec[12][0] = -beta; dodec[12][1] = alpha; dodec[12][2] = 0;
    dodec[13][0] = -beta; dodec[13][1] = -alpha; dodec[13][2] = 0;
    dodec[14][0] = -alpha; dodec[14][1] = 0; dodec[14][2] = -beta;
    dodec[15][0] = alpha; dodec[15][1] = 0; dodec[15][2] = -beta;
    dodec[16][0] = 0; dodec[16][1] = beta; dodec[16][2] = alpha;
    dodec[17][0] = 0; dodec[17][1] = beta; dodec[17][2] = -alpha;
    dodec[18][0] = 0; dodec[18][1] = -beta; dodec[18][2] = alpha;
    dodec[19][0] = 0; dodec[19][1] = -beta; dodec[19][2] = -alpha;
    /* *INDENT-ON* */

}

static void dodecahedron(GLenum type)
{
   static int inited = 0;

   if (inited == 0) {
      inited = 1;
      initDodecahedron();
   }
   pentagon(0, 1, 9, 16, 5, type);
   pentagon(1, 0, 3, 18, 7, type);
   pentagon(1, 7, 11, 10, 9, type);
   pentagon(11, 7, 18, 19, 6, type);
   pentagon(8, 17, 16, 9, 10, type);
   pentagon(2, 14, 15, 6, 19, type);
   pentagon(2, 13, 12, 4, 14, type);
   pentagon(2, 19, 18, 3, 13, type);
   pentagon(3, 0, 5, 12, 13, type);
   pentagon(6, 15, 8, 10, 11, type);
   pentagon(4, 17, 8, 15, 14, type);
   pentagon(4, 12, 5, 16, 17, type);
}

void glutSolidDodecahedron(void)
{
   dodecahedron(GL_TRIANGLE_FAN);
}
