/*
 * GLExcess OpenGL Benchmark
 * Version:  1.0
 *
 * Copyright (C) XXXX-2003  Paolo Martella   All Rights Reserved.
 * Portet to SDL by Mikael Brorsson.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included
 * in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
 * BRIAN PAUL BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN
 * AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
 * CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
*/

/*
 * main.cc
 * Main source file
 *
*/

#include <stdio.h>
#include <GL/gl.h>
#include <GL/glu.h>
#include <stdlib.h>
#include <math.h>
#include "SDL/SDL.h"
#include "Texture.h"
#include "utils.h"
#include "scene01.h"
#include "scene02.h"
#include "scene03.h"
#include "scene04.h"
#include "scene05.h"
#include "scene06.h"
#include "scene07.h"
#include "scene08.h"
#include "scene09.h"
#include "scene10.h"
#include "scene11.h"
#include "scene12.h"

#include "FMOD/fmod.h"
#include "FMOD/fmod_errors.h"

#define SCREEN_WIDTH 800
#define SCREEN_HEIGHT 600
#define SCREEN_DEPTH 24
#define FULLSCREEN 1

static bool loop = true; 							// Loop the demo after finish.
static bool timerset = false;

GLfloat timing = 0;
GLfloat step = 1;
GLuint scene = 1;

void KillGLWindow();
void HandleKeyPressEvent(SDL_keysym * keysym);

void ReSizeGLScene(GLsizei width, GLsizei height)
{
	if (height==0)
	{
		height=1;
	}

	glViewport(0,0,width,height);
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	gluPerspective(45.0f,(GLfloat)width/(GLfloat)height,0.1f,1000.0f);
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();
}

int InitGL(GLvoid)
{
	glShadeModel(GL_SMOOTH);
	glClearColor(0.0f, 0.0f, 0.0f, 0.5f);
	glClearDepth(1.0f);
	glEnable(GL_DEPTH_TEST);
	glDepthFunc(GL_LEQUAL);
	glHint(GL_PERSPECTIVE_CORRECTION_HINT, GL_NICEST);
	ReSizeGLScene(SCREEN_WIDTH,SCREEN_HEIGHT);
	return 1;
}

void KillGLWindow()
{
	SDL_Quit();
	exit(0);
}

int CreateGLWindow(char* title, int width, int height, int bits, int fullscreenflag)
{

	int VideoFlags = 0; 
	const SDL_VideoInfo* info = NULL;
	SDL_Surface * MainWindow = NULL;

	if( SDL_Init( SDL_INIT_VIDEO ) < 0 ) 
	{ 
		printf("Video initialization failed: %s\n",
		     SDL_GetError( ) );
		return 0;
	}

	info = SDL_GetVideoInfo( );

	if( !info )
	{
		printf("Video query failed: %s\n",
		     SDL_GetError( ) );		
		return 0;
    }

	SDL_GL_SetAttribute( SDL_GL_RED_SIZE, 5 );
    SDL_GL_SetAttribute( SDL_GL_GREEN_SIZE, 5 );
    SDL_GL_SetAttribute( SDL_GL_BLUE_SIZE, 5 );
    SDL_GL_SetAttribute( SDL_GL_DEPTH_SIZE, bits );
    SDL_GL_SetAttribute( SDL_GL_DOUBLEBUFFER, 1 );
	SDL_GL_SetAttribute( SDL_GL_STENCIL_SIZE, 0); 
	SDL_GL_SetAttribute( SDL_GL_ACCUM_RED_SIZE, 0);   
	SDL_GL_SetAttribute( SDL_GL_ACCUM_GREEN_SIZE, 0); 
	SDL_GL_SetAttribute( SDL_GL_ACCUM_BLUE_SIZE, 0);  
	SDL_GL_SetAttribute( SDL_GL_ACCUM_ALPHA_SIZE, 0); 
	
	SDL_WM_SetCaption(title,title);

	if(fullscreenflag)
	{
		VideoFlags   = SDL_OPENGL;    
		VideoFlags  |= SDL_HWPALETTE; 
		VideoFlags  |= SDL_RESIZABLE; 
		VideoFlags	|= SDL_FULLSCREEN;
	}
	else
	{
		VideoFlags   = SDL_OPENGL;   
		VideoFlags  |= SDL_HWPALETTE;
		VideoFlags  |= SDL_RESIZABLE;
		/*VideoFlags	|= SDL_NOFRAME;*/
	}

	if(info -> hw_available)
	{
		VideoFlags |= SDL_HWSURFACE;
	}
    else
	{
		VideoFlags |= SDL_SWSURFACE;
	}
	if(info -> blit_hw)
	{
        VideoFlags |= SDL_HWACCEL;
	}

	MainWindow = SDL_SetVideoMode( width, height, bits,VideoFlags);
	
	if(MainWindow == 0 ) 
	{
		
		return 0;
	}
	if (!InitGL())
	{
		printf("Init Failed\n");
		return 0;
	}

	return 1;
}

void drawscene()
{
	switch (scene)
	{
		case 1:
			if (!z_DrawGLScene(timing)) {scene++; z_Clean(); timing=0;}
			break;
		case 2:
			if (!a_DrawGLScene(timing)) {scene++; a_Clean(); timing=0;}
			break;
		case 3:
			if (!b_DrawGLScene(timing)) {scene++; b_Clean(); timing=0;}
			break;
		case 4:
			if (!d_DrawGLScene())		{scene++; d_Clean(); timing=0;}
			break;
		case 5:
			if (!c_DrawGLScene(timing)) {scene++; c_Clean(); timing=0;}
			break;
		case 6:
			if (!e_DrawGLScene(timing)) {scene++; e_Clean(); timing=0;}
			break;
		case 7:
			if (!f_DrawGLScene(timing)) {scene++; f_Clean(); timing=0;}
			break;
		case 8:
			if (!g_DrawGLScene(timing)) {scene++; g_Clean(); timing=0;}
			break;
		case 9:
			if (!h_DrawGLScene(timing)) {scene++; h_Clean(); timing=0;}
			break;
		case 10:
			if (!i_DrawGLScene(timing)) {scene++; i_Clean(); timing=0;}
			break;
		case 11:
			if (!j_DrawGLScene(timing)) {scene++; j_Clean(); timing=0;}
			break;
		case 12:
			if (!k_DrawGLScene(timing)) {scene++; k_Clean(); timing=0;}
			break;
		case 13:
			if (loop) {scene=1; timing=0;}
			else exit(0);
			break;
	}
}

void update(int dummy)
{
	timing+=3;
	timerset=false;
}

void scenemanagement()
{
	timing+=step;
	drawscene();
	//if (!timerset) {glutTimerFunc(1,update,0);timerset=true;}
}
void display(void)
{

	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	glOrtho(-1,1,-1,1,-1,1);
	glMatrixMode(GL_MODELVIEW);
	glClear (GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	glLoadIdentity();
	glPointSize(5);
	glEnable(GL_TEXTURE_2D);

	glBegin(GL_QUADS);
	glTexCoord2f(0,0);
	glVertex3f(-.5,-.5,0);
	glTexCoord2f(1,0);
	glVertex3f(.5,-.5,0);
	glTexCoord2f(1,1);
	glVertex3f(.5,.5,0);
	glTexCoord2f(0,1);
	glVertex3f(-.5,.5,0);
	glEnd();
	glColor4f(1,1,1,1);
	SDL_GL_SwapBuffers();
//	glutSwapBuffers();
}

void process_events()
{
    SDL_Event event;

    while( SDL_PollEvent( &event ) ) 
	{
        switch( event.type ) 
		{
        case SDL_KEYDOWN:
	        HandleKeyPressEvent( &event.key.keysym );
            break;

        case SDL_QUIT:
			KillGLWindow();
            break;
        }
    }
}

void HandleKeyPressEvent(SDL_keysym * keysym)
{
    switch(keysym->sym)
    {
        case SDLK_F1 :
            break;
		case SDLK_1:
			z_Clean(); 
		scene++;timing=0;
		break;
		case SDLK_2:
			a_Clean(); 
		scene++;timing=0;
		break;
		case SDLK_3:
			b_Clean(); 
		scene++;timing=0;
		break;
		case SDLK_4:
			d_Clean(); 
		scene++;timing=0;
		break;
		case SDLK_5:
			c_Clean(); 
		scene++;timing=0;
		break;
		case SDLK_6:
			e_Clean(); 
		scene++;timing=0;
		break;
		case SDLK_7:
			f_Clean(); 
		scene++;timing=0;
		break;
		case SDLK_8:
			g_Clean(); 
		scene++;timing=0;
		break;
		case SDLK_9:
			h_Clean(); 
		scene++;timing=0;
		break;
		case SDLK_0:
			i_Clean(); 
		scene++;timing=0;
		break;
		case SDLK_a:
			j_Clean(); 
		scene++;timing=0;
		break;
		case SDLK_b:
			k_Clean(); 
		scene++;timing=0;
		break;
		/*		case 2: a_Clean(); break;
				case 3: b_Clean(); break;
				case 4: d_Clean(); break;
				case 5: c_Clean(); break;
				case 6: e_Clean(); break;
				case 7: f_Clean(); break;
				case 8: g_Clean(); break;
				case 9: h_Clean(); break;
				case 10: i_Clean(); break;
				case 11: j_Clean(); break;
				case 12: k_Clean(); break;*/

        case SDLK_ESCAPE:
            KillGLWindow();
         
        default:
            break;
    }
	
}

int main(int argc, char** argv)
{
	int done = 0;
	int wparams[5]={0,SCREEN_WIDTH,SCREEN_HEIGHT,24,75};
	
	printf("argc=%d",argc);
	
	for (int i = 0; i<argc; i++)
	{
		printf("\nargv[%d]:%s",i,argv[i]);
	}
	
	for (int i = 0; ((i<argc-1)&&(i<5)); i++)
	{		
		wparams[i] = toint(argv[i+1]);	
	}
//	glutInit(&argc, argv);
//	glutInitDisplayMode (GLUT_DOUBLE | GLUT_RGB | GLUT_DEPTH);
	printf ("\ngamestring: %s\n",buildgamestring(wparams));
	if (!wparams[0])
	{
			CreateGLWindow("GLExcess",wparams[1],wparams[2], SCREEN_DEPTH, FULLSCREEN);
	//	glutInitWindowSize (wparams[1],wparams[2]);
	//	glutCreateWindow("GLExcess");
	}
	else
	{
		/*glutGameModeString(buildgamestring(wparams));
		if (glutGameModeGet(GLUT_GAME_MODE_POSSIBLE))
		{
			glutEnterGameMode();
		}
		else 
		{
			printf("\nSelected game mode not available\n");
			exit(1);
		}*/
	}
	while( !done ) 
	{
		process_events( );
		
        scenemanagement();
		//SDL_GL_SwapBuffers( );
    }
	//glutPostRedisplay();
	//glutKeyboardFunc(keyboard);
	//glutDisplayFunc(scenemanagement); 
	//glutIdleFunc(scenemanagement);
	
	//glutMainLoop();
	KillGLWindow();
	return 0;
}
